from fastapi import FastAPI, Query
from fastapi.middleware.cors import CORSMiddleware
from fastapi_contrib.conf import settings
from sqlalchemy import create_engine
from kafka import KafkaProducer
from pydantic import BaseModel, Field
from typing import Optional, List
from jaeger_client import Config
from opentracing.scope_managers.asyncio import AsyncioScopeManager
from prometheus_client import Counter
from prometheus_fastapi_instrumentator import Instrumentator, metrics
from prometheus_fastapi_instrumentator.metrics import Info

import uvicorn
import time
import json


app = FastAPI(title='Offer Service API')
db = create_engine('postgresql://postgres:postgres@offer_service_db:5432/postgres')
kafka_producer = None

app.add_middleware(
    CORSMiddleware,
    allow_origins=['http://localhost:4200'],
    allow_credentials=True,
    allow_methods=['*'],
    allow_headers=['*'],
)

def setup_opentracing(app):
    config = Config(
        config={
            "local_agent": {
                "reporting_host": settings.jaeger_host,
                "reporting_port": settings.jaeger_port
            },
            "sampler": {
                "type": settings.jaeger_sampler_type,
                "param": settings.jaeger_sampler_rate,
            },
            "trace_id_header": settings.trace_id_header
        },
        service_name="offer_service",
        validate=True,
        scope_manager=AsyncioScopeManager()
    )

    app.state.tracer = config.initialize_tracer()
    app.tracer = app.state.tracer

setup_opentracing(app)

def http_404_requests():
    METRIC = Counter(
        "http_404_requests",
        "Number of times a 404 request has occured.",
        labelnames=("path",)
    )

    def instrumentation(info: Info):
        if info.response.status_code == 404:
            METRIC.labels(info.request.url).inc()

    return instrumentation

def http_unique_users():
    METRIC = Counter(
        "http_unique_users",
        "Number of unique http users.",
        labelnames=("user",)
    )

    def instrumentation(info: Info):
        try:
            user = f'{info.request.client.host} {info.request.headers["User-Agent"]}'
        except:
            user = f'{info.request.client.host} Unknown'
        METRIC.labels(user).inc()

    return instrumentation


instrumentator = Instrumentator(excluded_handlers=["/metrics"])
instrumentator.add(metrics.default())
instrumentator.add(metrics.combined_size())
instrumentator.add(http_404_requests())
instrumentator.add(http_unique_users())
instrumentator.instrument(app).expose(app)

OFFERS_URL = '/api/offers'


class Offer(BaseModel):
    id: Optional[int] = Field(description='Offer ID')
    position: str = Field(description='Offer Position', min_length=1)
    requirements: str = Field(description='Offer Requirements', min_length=1)
    description: str = Field(description='Offer Description', min_length=1)
    agent_application_link: str = Field(description='Offer Agent Application Link', min_length=1)


class NavigationLinks(BaseModel):
    base: str = Field('http://localhost:9000/api', description='API base URL')
    prev: Optional[str] = Field(None, description='Link to the previous page')
    next: Optional[str] = Field(None, description='Link to the next page')


class Response(BaseModel):
    results: List[Offer]
    links: NavigationLinks
    offset: int
    limit: int
    size: int

def register_kafka_producer():
    global kafka_producer
    while True:
        try:
            kafka_producer = KafkaProducer(bootstrap_servers='kafka:9092', value_serializer=lambda v: json.dumps(v).encode('utf-8'))
            break
        except:
            time.sleep(3)   

def record_action(status: int, message: str, span):
    print("{0:10}{1}".format('ERROR:' if status >= 400 else 'INFO:', message))
    span.set_tag('http_status', status)
    span.set_tag('message', message)

def record_event(type: str, data: dict):
    kafka_producer.send('events', {
        'type': type,
        'data': data
    })

@app.post(OFFERS_URL)
def create_offer(offer: Offer):
    with app.tracer.start_span('Create Offer Request') as span:
        try:
            span.set_tag('http_method', 'POST')
            db.execute('insert into offers (position, requirements, description, agent_application_link) values (%s, %s, %s, %s)', 
                                (offer.position, offer.requirements, offer.description, offer.agent_application_link))

            record_action(200, 'Request successful', span)
            record_event('Job Offer Created', dict(offer))
        except Exception as e:
            record_action(500, 'Request failed', span)
            raise e


def search_query(search: str):
    return 'lower(position) like %s or lower(requirements) like %s or lower(description) like %s', (f'%{search.lower()}%', f'%{search.lower()}%', f'%{search.lower()}%')


@app.get(OFFERS_URL)
def read_offers(search: str = Query(''), offset: int = Query(0), limit: int = Query(7)):
    with app.tracer.start_span('Read Offers Request') as span:
        try:
            span.set_tag('http_method', 'GET')
            query, params = search_query(search)
            total_offers = len(list(db.execute(f'select * from offers where {query}', params)))
            offers = db.execute(f'select * from offers where {query} order by id desc offset {offset} limit {limit}', params)

            prev_link = f'/offers?search={search}&offset={offset-limit}&limit={limit}' if offset - limit >= 0 else None
            next_link = f'/offers?search={search}&offset={offset+limit}&limit={limit}' if offset + limit < total_offers else None
            links = NavigationLinks(prev=prev_link, next=next_link)
            results = [Offer.parse_obj(offer) for offer in offers]

            record_action(200, 'Request successful', span)
            return Response(results=results, links=links, offset=offset, limit=limit, size=len(results))
        except Exception as e:
            record_action(500, 'Request failed', span)
            raise e


def run_service():
    register_kafka_producer()
    uvicorn.run(app, host='0.0.0.0', port=8002)


if __name__ == '__main__':
    run_service()



from sqlalchemy import create_engine

import pytest
import requests
import time
import json

db = create_engine('postgresql://postgres:postgres@localhost:5435/postgres')
OFFERS_URL = 'http://localhost:8002/api/offers'

@pytest.fixture(scope='session', autouse=True)
def do_something(request):
    time.sleep(30)


def test_create_offer_missing_position():
    data = {
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'position'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_position():
    data = {
        'position': None,
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'position'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_position():
    data = {
        'position': '',
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'position'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

def test_create_offer_missing_requirements():
    data = {
        'position': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'requirements'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_requirements():
    data = {
        'position': 'asd',
        'requirements': None,
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'requirements'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_requirements():
    data = {
        'position': 'asd',
        'requirements': '',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'requirements'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

def test_create_offer_missing_description():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'description'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_description():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': None,
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'description'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_description():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': '',
        'agent_application_link': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'description'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

def test_create_offer_missing_agent_application_link():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': 'asd'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'agent_application_link'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_agent_application_link():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': None
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'agent_application_link'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_agent_application_link():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': ''
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'agent_application_link'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1


def reset_table(number_of_rows=0):
    db.execute('delete from offers')
    for i in range(number_of_rows):
        db.execute('''
            insert into offers (position, requirements, description, agent_application_link)
            values (%s, %s, %s, %s)
        ''', (f'position {i+1}', f'requirements {i+1}', f'description {i+1}', f'agent_application_link {i+1}'))


def test_create_offer_valid():
    reset_table()
    data = {
        'position': 'position 1',
        'requirements': 'requirements 1',
        'description': 'description 1',
        'agent_application_link': 'agent_application_link 1'
    }
    res = requests.post(OFFERS_URL, json=data)
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body is None
    offers = list(db.execute('select * from offers'))
    assert len(offers) == 1
    assert offers[0]['position'] == 'position 1'
    assert offers[0]['requirements'] == 'requirements 1'
    assert offers[0]['description'] == 'description 1'
    assert offers[0]['agent_application_link'] == 'agent_application_link 1'

def test_read_offers():
    reset_table(10)
    res = requests.get(OFFERS_URL)
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] is None
    assert body['links']['next'] == '/offers?search=&offset=7&limit=7'
    assert body['offset'] == 0
    assert body['limit'] == 7
    assert body['size'] == 7
    assert len(body['results']) == 7
    for i in range(7):
        assert body['results'][i]['position'] == f'position {10-i}'
        assert body['results'][i]['requirements'] == f'requirements {10-i}'
        assert body['results'][i]['description'] == f'description {10-i}'
        assert body['results'][i]['agent_application_link'] == f'agent_application_link {10-i}'

def test_read_offers_with_offset():
    reset_table(10)
    res = requests.get(f'{OFFERS_URL}?offset=7')
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] == '/offers?search=&offset=0&limit=7'
    assert body['links']['next'] is None
    assert body['offset'] == 7
    assert body['limit'] == 7
    assert body['size'] == 3
    assert len(body['results']) == 3
    
def test_read_offers_with_limit():
    reset_table(10)
    res = requests.get(f'{OFFERS_URL}?limit=10')
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] is None
    assert body['links']['next'] is None
    assert body['offset'] == 0
    assert body['limit'] == 10
    assert body['size'] == 10
    assert len(body['results']) == 10
    for i in range(10):
        assert body['results'][i]['position'] == f'position {10-i}'
        assert body['results'][i]['requirements'] == f'requirements {10-i}'
        assert body['results'][i]['description'] == f'description {10-i}'
        assert body['results'][i]['agent_application_link'] == f'agent_application_link {10-i}'

def test_search_offers_by_position():
    reset_table(10)
    res = requests.get(f'{OFFERS_URL}?search=POSITION')
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] is None
    assert body['links']['next'] == '/offers?search=POSITION&offset=7&limit=7'
    assert body['offset'] == 0
    assert body['limit'] == 7
    assert body['size'] == 7
    assert len(body['results']) == 7
    for i in range(7):
        assert body['results'][i]['position'] == f'position {10-i}'
        assert body['results'][i]['requirements'] == f'requirements {10-i}'
        assert body['results'][i]['description'] == f'description {10-i}'
        assert body['results'][i]['agent_application_link'] == f'agent_application_link {10-i}'

def test_search_offers_by_requirements():
    reset_table(10)
    res = requests.get(f'{OFFERS_URL}?search=REQUIREMENTS')
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] is None
    assert body['links']['next'] == '/offers?search=REQUIREMENTS&offset=7&limit=7'
    assert body['offset'] == 0
    assert body['limit'] == 7
    assert body['size'] == 7
    assert len(body['results']) == 7
    for i in range(7):
        assert body['results'][i]['position'] == f'position {10-i}'
        assert body['results'][i]['requirements'] == f'requirements {10-i}'
        assert body['results'][i]['description'] == f'description {10-i}'
        assert body['results'][i]['agent_application_link'] == f'agent_application_link {10-i}'

def test_search_offers_by_description():
    reset_table(10)
    res = requests.get(f'{OFFERS_URL}?search=DESCRIPTION')
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] is None
    assert body['links']['next'] == '/offers?search=DESCRIPTION&offset=7&limit=7'
    assert body['offset'] == 0
    assert body['limit'] == 7
    assert body['size'] == 7
    assert len(body['results']) == 7
    for i in range(7):
        assert body['results'][i]['position'] == f'position {10-i}'
        assert body['results'][i]['requirements'] == f'requirements {10-i}'
        assert body['results'][i]['description'] == f'description {10-i}'
        assert body['results'][i]['agent_application_link'] == f'agent_application_link {10-i}'

def test_search_offer_by_agent_application_link():
    reset_table(10)
    res = requests.get(f'{OFFERS_URL}?search=AGENT_APPLICATION_LINK')
    assert res.status_code == 200
    body = json.loads(res.text)
    assert body['links']['prev'] is None
    assert body['links']['next'] is None
    assert body['offset'] == 0
    assert body['limit'] == 7
    assert body['size'] == 0
    assert len(body['results']) == 0



from offer_service.main import app, OFFERS_URL
from fastapi.testclient import TestClient
from mock import patch
import json

class TestDB:
    def execute(self, sql, params):
        return []
    
class TestKP:
    def send(self, topic, data):
        pass

client = TestClient(app)
testDB = TestDB()
testKP = TestKP()

def fix_whitespaces(text: str):
    return ' '.join(text.split())

def test_create_offer_missing_position():
    data = {
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'position'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_position():
    data = {
        'position': None,
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'position'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_position():
    data = {
        'position': '',
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'position'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

def test_create_offer_missing_requirements():
    data = {
        'position': 'asd',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'requirements'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_requirements():
    data = {
        'position': 'asd',
        'requirements': None,
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'requirements'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_requirements():
    data = {
        'position': 'asd',
        'requirements': '',
        'description': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'requirements'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

def test_create_offer_missing_description():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'description'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_description():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': None,
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'description'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_description():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': '',
        'agent_application_link': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'description'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

def test_create_offer_missing_agent_application_link():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': 'asd'
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'agent_application_link'
    assert body['detail'][0]['msg'] == 'field required'
    assert body['detail'][0]['type'] == 'value_error.missing'

def test_create_offer_null_agent_application_link():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': None
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'agent_application_link'
    assert body['detail'][0]['msg'] == 'none is not an allowed value'
    assert body['detail'][0]['type'] == 'type_error.none.not_allowed'

def test_create_offer_empty_agent_application_link():
    data = {
        'position': 'asd',
        'requirements': 'asd',
        'description': 'asd',
        'agent_application_link': ''
    }
    res = client.post(OFFERS_URL, json=data)
    assert res.status_code == 422
    body = json.loads(res.text)
    assert body['detail'][0]['loc'][0] == 'body'
    assert body['detail'][0]['loc'][1] == 'agent_application_link'
    assert body['detail'][0]['msg'] == 'ensure this value has at least 1 characters'
    assert body['detail'][0]['type'] == 'value_error.any_str.min_length'
    assert body['detail'][0]['ctx']['limit_value'] == 1

@patch('offer_service.main.db', testDB)
@patch('offer_service.main.kafka_producer', testKP)
def test_create_offer_valid():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        data = {
            'position': 'position 1',
            'requirements': 'requirements 1',
            'description': 'description 1',
            'agent_application_link': 'agent_application_link 1'
        }
        res = client.post(OFFERS_URL, json=data)
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body is None
        spy.assert_called_once()
        spy.assert_called_once_with('insert into offers (position, requirements, description, agent_application_link) values (%s, %s, %s, %s)', ('position 1', 'requirements 1', 'description 1', 'agent_application_link 1'))

@patch('offer_service.main.db', testDB)
def test_read_offers():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(OFFERS_URL)
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] is None
        assert body['links']['next'] is None
        assert body['offset'] == 0
        assert body['limit'] == 7
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 0 limit 7
        '''), ('%%', '%%', '%%'))

@patch('offer_service.main.db', testDB)
def test_read_offers_with_offset():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(f'{OFFERS_URL}?offset=7')
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] == '/offers?search=&offset=0&limit=7'
        assert body['links']['next'] is None
        assert body['offset'] == 7
        assert body['limit'] == 7
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 7 limit 7
        '''), ('%%', '%%', '%%'))

@patch('offer_service.main.db', testDB)
def test_read_offers_with_limit():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(f'{OFFERS_URL}?limit=10')
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] is None
        assert body['links']['next'] is None
        assert body['offset'] == 0
        assert body['limit'] == 10
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 0 limit 10
        '''), ('%%', '%%', '%%'))

@patch('offer_service.main.db', testDB)
def test_search_offers_by_position():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(f'{OFFERS_URL}?search=POSITION')
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] is None
        assert body['links']['next'] is None
        assert body['offset'] == 0
        assert body['limit'] == 7
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 0 limit 7
        '''), ('%position%', '%position%', '%position%'))

@patch('offer_service.main.db', testDB)
def test_search_offers_by_requirements():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(f'{OFFERS_URL}?search=REQUIREMENTS')
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] is None
        assert body['links']['next'] is None
        assert body['offset'] == 0
        assert body['limit'] == 7
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 0 limit 7
        '''), ('%requirements%', '%requirements%', '%requirements%'))

@patch('offer_service.main.db', testDB)
def test_search_offers_by_description():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(f'{OFFERS_URL}?search=DESCRIPTION')
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] is None
        assert body['links']['next'] is None
        assert body['offset'] == 0
        assert body['limit'] == 7
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 0 limit 7
        '''), ('%description%', '%description%', '%description%'))

@patch('offer_service.main.db', testDB)
def test_search_offer_by_agent_application_link():
    with patch.object(testDB, 'execute', wraps=testDB.execute) as spy:        
        res = client.get(f'{OFFERS_URL}?search=AGENT_APPLICATION_LINK')
        assert res.status_code == 200
        body = json.loads(res.text)
        assert body['links']['prev'] is None
        assert body['links']['next'] is None
        assert body['offset'] == 0
        assert body['limit'] == 7
        assert body['size'] == 0
        assert len(body['results']) == 0
        spy.assert_called()
        spy.assert_called_with(fix_whitespaces('''
            select * from offers
            where lower(position) like %s or lower(requirements) like %s or lower(description) like %s
            order by id desc offset 0 limit 7
        '''), ('%agent_application_link%', '%agent_application_link%', '%agent_application_link%'))
